@@ -159,7 +159,7 @@ get_merge_type (const char *str)
 		return MERGE_BOOLEAN;
 	if (strcmp (str, "int") == 0)
 		return MERGE_INT32;
-	if (strcmp (str, "unint64") == 0)
+	if (strcmp (str, "uint64") == 0)
 		return MERGE_UINT64;
 	if (strcmp (str, "double") == 0)
 		return MERGE_DOUBLE;
@@ -305,15 +305,17 @@ static void remember_jump_position(struct fdi_context *fdi_ctx)
 static void set_jump_position(struct fdi_context *fdi_ctx)
 {
 	off_t	offset;
+	u_int32_t offset32;
 
 	if (fdi_ctx->depth <= 0)
 		DIE(("Rule depth underrun"));
 
 	fdi_ctx->depth--;
 	offset = RULES_ROUND(lseek(fdi_ctx->cache_fd, 0, SEEK_END));
+	offset32 = (u_int32_t)offset;
 	pad32_write(fdi_ctx->cache_fd,
 		fdi_ctx->match_at_depth[fdi_ctx->depth] + offsetof(struct rule, jump_position),
-		&offset, sizeof(fdi_ctx->rule.jump_position));
+		&offset32, sizeof(fdi_ctx->rule.jump_position));
 
 	if (haldc_verbose)
 		HAL_INFO(("modify rule=0x%08x, set jump to 0x%08x",
@@ -477,6 +479,7 @@ rules_add_fdi_file (const char *filename, int fd)
 	parser = XML_ParserCreate (NULL);
 	if (parser == NULL) {
 		HAL_ERROR (("Couldn't allocate memory for parser"));
+		g_free (fdi_ctx);
 		goto out;
 	}
 	XML_SetUserData (parser, fdi_ctx);
@@ -700,6 +703,7 @@ di_rules_init (void)
 		HAL_INFO (("Generating rules done (occupying %d bytes)", header.all_rules_size));
 	}
 
+	g_free (cachename_temp);
 	return num_skipped_fdi_files;
 error:
 	HAL_ERROR (("Error generating fdi cache"));
