@@ -27,7 +27,7 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <bitstring.h>
+#include <sys/bitstring.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <unistd.h>
@@ -53,6 +53,7 @@
 #define PCIR_HDRTYPE			0x0e
 #define PCIM_HDRTYPE_BRIDGE		0x01
 #define PCIM_HDRTYPE_CARDBUS		0x02
+#define PCIM_MFDEV                      0x80
 
 typedef struct
 {
@@ -100,7 +101,6 @@ hf_pci_device_new (HalDevice *parent, const struct pci_conf *p, int secondary_bu
 
   hf_device_set_udi(device, "pci_%.4x_%.4x", p->pc_vendor, p->pc_device);
   hal_device_property_set_string(device, "info.subsystem", "pci");
-  hal_device_property_set_string(device, "info.bus", "pci");
   hal_device_property_set_int(device, "pci.device_class", p->pc_class);
   hal_device_property_set_int(device, "pci.device_subclass", p->pc_subclass);
   hal_device_property_set_int(device, "pci.device_protocol", p->pc_progif);
@@ -250,7 +250,7 @@ hf_pci_probe_bus (HalDevice *parent, int bus, bitstr_t *busmap)
             continue;	/* device ignored */
 	}
 
-      if (info->header_type == PCIM_HDRTYPE_BRIDGE || info->header_type == PCIM_HDRTYPE_CARDBUS)
+      if (info->header_type == PCIM_HDRTYPE_BRIDGE || info->header_type == PCIM_HDRTYPE_CARDBUS || (info->header_type & ~PCIM_MFDEV) == PCIM_HDRTYPE_BRIDGE || (info->header_type & ~PCIM_MFDEV) == PCIM_HDRTYPE_CARDBUS)
         /* a bridge or cardbus, probe its children */
         hf_pci_probe_bus(info->device, info->secondary_bus, busmap);
     }
