@@ -25,13 +25,20 @@
 #  include <config.h>
 #endif
 
+#include <sys/param.h>
 #include <string.h>
 #include <errno.h>
 #include <stdlib.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
 #include <unistd.h>
+#if __FreeBSD_version < 800092
+#if __FreeBSD_version >= 800064
+#include <legacy/dev/usb/usb.h>
+#else
 #include <dev/usb/usb.h>
+#endif
+#endif
 
 #include "../logger.h"
 #include "../osspec.h"
@@ -40,7 +47,13 @@
 #include "hf-devtree.h"
 #include "hf-util.h"
 
+#if __FreeBSD_version < 800092
 #define HF_USB_DEVICE			"/dev/usb"
+#if __FreeBSD_version < 800066
+#define HF_USB2_DEVICE			"/dev/usb "
+#else
+#define HF_USB2_DEVICE			"/dev/usbctl"
+#endif
 
 typedef struct
 {
@@ -227,31 +240,6 @@ hf_usb_get_full_config_descriptor (int fd,
   return NULL;
 }
 
-/*
- * Adapted from usb_compute_udi() in linux2/physdev.c and
- * usbclass_compute_udi() in linux2/classdev.c.
- */
-static void
-hf_usb_device_compute_udi (HalDevice *device)
-{
-  g_return_if_fail(HAL_IS_DEVICE(device));
-
-  if (hal_device_has_capability(device, "hiddev"))
-    hf_device_set_full_udi(device, "%s_hiddev",
-			   hal_device_property_get_string(device, "info.parent"));
-  else if (hal_device_has_property(device, "usb.interface.number"))
-    hf_device_set_full_udi(device, "%s_if%i",
-			   hal_device_property_get_string(device, "info.parent"),
-			   hal_device_property_get_int(device, "usb.interface.number"));
-  else
-    hf_device_set_udi(device, "usb_device_%x_%x_%s",
-		      hal_device_property_get_int(device, "usb_device.vendor_id"),
-		      hal_device_property_get_int(device, "usb_device.product_id"),
-		      hal_device_has_property(device, "usb_device.serial")
-		      ? hal_device_property_get_string(device, "usb_device.serial")
-		      : "noserial");
-}
-
 /* adapted from usbif_set_name() in linux2/physdev.c */
 static const char *
 hf_usb_get_interface_name (const usb_interface_descriptor_t *desc)
@@ -315,8 +303,6 @@ hf_usb_device_new (HalDevice *parent,
   device = hf_device_new(parent);
 
   hal_device_property_set_string(device, "info.subsystem", "usb_device");
-  hal_device_property_set_string(device, "info.bus", "usb_device");
-
   hal_device_property_set_string(device, "info.product", di->udi_product);
   hal_device_property_set_string(device, "info.vendor", di->udi_vendor);
 
@@ -398,10 +384,17 @@ hf_usb_device_new (HalDevice *parent,
 	g_free(port);
       }
 
+  /*
+   * Register the first attached driver (if any) with devtree (mostly
+   * useful for allowing hf-scsi to find umass devices).
+   */
+  if (*di->udi_devnames[0])
+    hf_devtree_device_set_name(device, di->udi_devnames[0]);
+
   if ((devname = hf_usb_get_devname(di, "ukbd")))	/* USB keyboard */
-    hf_device_set_input(device, "keyboard", devname);
+    hf_device_set_input(device, "keyboard", "keys", devname);
   else if ((devname = hf_usb_get_devname(di, "ums")))	/* USB mouse */
-    hf_device_set_input(device, "mouse", devname);
+    hf_device_set_input(device, "mouse", NULL, devname);
   else if ((devname = hf_usb_get_devname(di, "uhid")))	/* UHID device */
     {
       hal_device_property_set_string(device, "info.category", "hiddev");
@@ -409,13 +402,20 @@ hf_usb_device_new (HalDevice *parent,
       hf_device_property_set_string_printf(device, "hiddev.device", "/dev/%s", devname);
       hal_device_copy_property(device, "info.product", device, "hiddev.product");
     }
-
-  /*
-   * Register the first attached driver (if any) with devtree (mostly
-   * useful for allowing hf-scsi to find umass devices).
-   */
-  if (*di->udi_devnames[0])
-    hf_devtree_device_set_name(device, di->udi_devnames[0]);
+  else if ((devname = hf_usb_get_devname(di, "ldev")))	/* Linux driver (webcam) */
+    {
+      /*
+       * XXX This is a hack.  Currently, all ldev devices are webcams.  However,
+       * that may not always be the case.  Hopefully, when other Linux driver
+       * support is added, there will be a sysctl or some other way to
+       * determine device class.
+       */
+      hf_usb_add_webcam_properties(device);
+    }
+  else if ((devname = hf_usb_get_devname(di, "pwc")))	/* Phillips Web Cam */
+    {
+      hf_usb_add_webcam_properties(device);
+    }
 
   hf_usb_device_compute_udi(device);
 
@@ -436,7 +436,7 @@ hf_usb_interface_device_new (HalDevice *parent,
 
   device = hf_device_new(parent);
 
-  hal_device_property_set_string(device, "info.bus", "usb");
+  hal_device_property_set_string(device, "info.subsystem", "usb");
 
   hal_device_merge_with_rewrite(device, parent, "usb.", "usb_device.");
 
@@ -543,6 +543,8 @@ hf_usb_probe_device (HalDevice *parent,
     {
       if (hal_device_has_capability(device, "hiddev"))
 	hf_runner_run_sync(device, 0, "hald-probe-hiddev", NULL);
+      if (hal_device_has_capability(device, "input.mouse"))
+        hf_runner_run_sync(device, 0, "hald-probe-mouse", NULL);
 
       hf_device_add(device);
     }
@@ -601,9 +603,18 @@ hf_usb_privileged_init (void)
 {
   int i;
 
+  if (g_file_test(HF_USB2_DEVICE, G_FILE_TEST_EXISTS))
+    {
+      hf_usb_fd = -1;
+      return;
+    }
+
   hf_usb_fd = open(HF_USB_DEVICE, O_RDONLY);
   if (hf_usb_fd < 0)
-    HAL_INFO(("unable to open %s: %s", HF_USB_DEVICE, g_strerror(errno)));
+    {
+      HAL_INFO(("unable to open %s: %s", HF_USB_DEVICE, g_strerror(errno)));
+      return;
+    }
 
   for (i = 0; i < 16; i++)
     {
@@ -853,3 +864,50 @@ HFDevdHandler hf_usb_devd_handler = {
   .add =	hf_usb_devd_add,
   .remove =	hf_usb_devd_remove
 };
+#endif
+
+/*
+ * Adapted from usb_compute_udi() in linux2/physdev.c and
+ * usbclass_compute_udi() in linux2/classdev.c.
+ */
+void
+hf_usb_device_compute_udi (HalDevice *device)
+{
+  g_return_if_fail(HAL_IS_DEVICE(device));
+
+  if (hal_device_has_capability(device, "hiddev"))
+    hf_device_set_full_udi(device, "%s_hiddev",
+			   hal_device_property_get_string(device, "info.parent"));
+  else if (hal_device_has_capability(device, "video4linux"))
+    hf_device_set_full_udi(device, "%s_video4linux",
+		    	   hal_device_property_get_string(device, "info.parent"));
+  else if (hal_device_has_property(device, "usb.interface.number"))
+    hf_device_set_full_udi(device, "%s_if%i",
+			   hal_device_property_get_string(device, "info.parent"),
+			   hal_device_property_get_int(device, "usb.interface.number"));
+  else
+    hf_device_set_udi(device, "usb_device_%x_%x_%s",
+		      hal_device_property_get_int(device, "usb_device.vendor_id"),
+		      hal_device_property_get_int(device, "usb_device.product_id"),
+		      (hal_device_has_property(device, "usb_device.serial") &&
+		       strcmp(hal_device_property_get_string(device, "usb_device.serial"), ""))
+		      ? hal_device_property_get_string(device, "usb_device.serial")
+		      : "noserial");
+}
+
+void
+hf_usb_add_webcam_properties (HalDevice *device)
+{
+  int unit;
+
+  g_return_if_fail(HAL_IS_DEVICE(device));
+
+  unit = hal_device_property_get_int(device, "freebsd.unit");
+  if (unit < 0)
+    unit = 0;
+
+  hal_device_property_set_string(device, "info.category", "video4linux");
+  hal_device_add_capability(device, "video4linux");
+  hf_device_property_set_string_printf(device, "video4linux.device", "/dev/video%i", unit);
+  hal_device_property_set_string(device, "info.product", "Video Device");
+}
