@@ -140,6 +140,8 @@ runner_server_message_handler (DBusConnection * connection,
 					break;
 				}
 			}
+		} else {
+			dbus_error_free (&error);
 		}
 	} else if (dbus_message_is_signal (message,
 					   DBUS_INTERFACE_LOCAL,
@@ -255,7 +257,8 @@ hald_runner_start_runner (void)
 	dbus_error_init (&err);
 	runner_server = dbus_server_listen (DBUS_SERVER_ADDRESS, &err);
 	if (runner_server == NULL) {
-		HAL_ERROR (("Cannot create D-BUS server for the runner"));
+		HAL_ERROR (("Cannot create D-BUS server for the runner: %s:%s", err.name, err.message));
+		dbus_error_free (&err);
 		goto error;
 	}
 
@@ -402,7 +405,7 @@ add_basic_env (DBusMessageIter * iter, const gchar * udi)
 			GSList *j;
 			CKSeat *seat;
 			GSList *sessions;
-			const char *seat_id;
+			char *seat_id;
 			GString *sessions_string;
                         int num_sessions;
 
@@ -424,7 +427,7 @@ add_basic_env (DBusMessageIter * iter, const gchar * udi)
                         num_sessions_total += num_sessions;
 			for (j = sessions; j != NULL; j = g_slist_next (j)) {
 				CKSession *session;
-				const char *session_id;
+				char *session_id;
                                 const char *q;
 
 				session = j->data;
@@ -451,7 +454,7 @@ add_basic_env (DBusMessageIter * iter, const gchar * udi)
 				add_env (iter, s, ck_session_is_active (session) ? "true" : "false");
 				g_free (s);
 				s = g_strdup_printf ("CK_SESSION_UID_%s", session_id);
-				p = g_strdup_printf ("%d", ck_session_get_user (session));
+				p = g_strdup_printf ("%u", ck_session_get_user (session));
 				add_env (iter, s, p);
 				g_free (s);
 				g_free (p);
@@ -464,6 +467,7 @@ add_basic_env (DBusMessageIter * iter, const gchar * udi)
                                         add_env (iter, s, q);
                                         g_free (s);
                                 }
+				g_free (session_id);
 			}
 
 			/* for each Seat, export sessions on each seat 
@@ -481,6 +485,7 @@ add_basic_env (DBusMessageIter * iter, const gchar * udi)
                         add_env (iter, s, p);
                         g_free (p);
 			g_free (s);
+			g_free (seat_id);
 
 		}
 
@@ -510,7 +515,7 @@ add_basic_env (DBusMessageIter * iter, const gchar * udi)
 	}
 #endif /* HAVE_CONKIT */
 
-	if (uname (&un) == 0) {
+	if (uname (&un) >= 0) {
 		char *sysname;
 
 		sysname = g_ascii_strdown (un.sysname, -1);
@@ -626,9 +631,8 @@ runner_start (HalDevice * device, const gchar * command_line,
 	}
 
 	/* Wait for the reply, should be almost instantanious */
-	reply =
-	    dbus_connection_send_with_reply_and_block (runner_connection,
-						       msg, -1, &error);
+	reply = dbus_connection_send_with_reply_and_block (runner_connection,
+							   msg, -1, &error);
 	if (reply) {
 		gboolean ret =
 		    (dbus_message_get_type (reply) ==
@@ -660,6 +664,7 @@ runner_start (HalDevice * device, const gchar * command_line,
 				}
 			} else {
 				HAL_ERROR (("Error extracting out_pid from runner's Start()"));
+				dbus_error_free (&error);
 			}
 		}
 
@@ -669,10 +674,11 @@ runner_start (HalDevice * device, const gchar * command_line,
 	} else {
 		if (dbus_error_is_set (&error)) {
 			HAL_ERROR (("Error running '%s': %s: %s", command_line, error.name, error.message));
+			dbus_error_free (&error);
 		}
 	}
 
-      error:
+error:
 	dbus_message_unref (msg);
 	return FALSE;
 }
@@ -880,6 +886,7 @@ hald_runner_run_sync (HalDevice * device,
 	if (reply == NULL) {
 		if (dbus_error_is_set (&error)) {
 			HAL_ERROR (("Error running '%s': %s: %s", command_line, error.name, error.message));
+			dbus_error_free (&error);
 		}
 		goto error;
 	}
@@ -927,18 +934,19 @@ hald_runner_kill_device (HalDevice * device)
 
 	/* Wait for the reply, should be almost instantanious */
 	dbus_error_init (&err);
-	reply =
-	    dbus_connection_send_with_reply_and_block (runner_connection,
-						       msg, -1, &err);
+	reply = dbus_connection_send_with_reply_and_block (runner_connection,
+							   msg, -1, &err);
 	if (reply) {
 		dbus_message_unref (reply);
+	} else {
+		dbus_error_free (&err);
 	}
 
 	dbus_message_unref (msg);
 }
 
 void
-hald_runner_kill_all ()
+hald_runner_kill_all (void)
 {
 	DBusMessage *msg, *reply;
 	DBusError err;
@@ -957,6 +965,8 @@ hald_runner_kill_all ()
 						       msg, -1, &err);
 	if (reply) {
 		dbus_message_unref (reply);
+	} else {
+		dbus_error_free (&err);
 	}
 
 	dbus_message_unref (msg);
