@@ -23,7 +23,11 @@
  *
  **************************************************************************/
 
-#include <config.h>
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
+#include <string.h>
 
 #include <glib/gmain.h>
 #include <dbus/dbus-glib.h>
@@ -44,6 +48,10 @@ static const int DELL_LCD_BRIGHTNESS_TOKEN = 0x007d;
 static u32 minValue = 0;
 static u32 maxValue = 0;
 
+#ifdef HAVE_SMBIOS_2_0_3
+static bool hasPass = false;
+#endif
+
 using namespace std;
 using namespace smi;
 
@@ -85,6 +93,15 @@ read_backlight (dbus_bool_t onAC)
 	return curValue;
 }
 
+#ifdef HAVE_SMBIOS_2_0_3
+static void
+check_bios_password()
+{
+	if (smi::getPasswordStatus((u16)9) || smi::getPasswordStatus((u16)10))
+		hasPass=true;
+}
+#endif
+
 static void 
 write_backlight (u32 newBacklightValue, dbus_bool_t onAC) 
 {	
@@ -92,6 +109,13 @@ write_backlight (u32 newBacklightValue, dbus_bool_t onAC)
 	u32 curValue;
 	writefn writeFunction;
 	string password(""); /* FIXME: Implement password support */
+
+#ifdef HAVE_SMBIOS_2_0_3
+	if (hasPass) {
+		HAL_WARNING(("Setting brightness via SMI is not supported with a BIOS password"));
+		return;
+	}
+#endif
 	
 	if (onAC) 
                 writeFunction = &smi::writeACModeSetting;
@@ -161,6 +185,7 @@ check_priv (DBusConnection *connection, DBusMessage *message, const char *udi, c
         ret = TRUE;
 
 out:
+	LIBHAL_FREE_DBUS_ERROR (&error);
         if (polkit_result != NULL)
                 libhal_free_string (polkit_result);
         return ret;
@@ -177,17 +202,29 @@ filter_function (DBusConnection *connection, DBusMessage *message, void *userdat
 	DBusError err;
 	DBusMessage *reply = NULL;
 	dbus_bool_t AC;
+	char **udis;
+	int i, num_devices;
+
 	dbus_error_init (&err);
 
         if (!check_priv (connection, message, dbus_message_get_path (message), "org.freedesktop.hal.power-management.lcd-panel")) {
                 return DBUS_HANDLER_RESULT_HANDLED;
         }
-
+	
+	/* set a default */
+	AC = TRUE; 
 	/* Mechanism to ensure that we always set the AC brightness when we are on AC-power etc. */
-	AC = libhal_device_get_property_bool (halctx, 
-					     "/org/freedesktop/Hal/devices/acpi_AC",
-					     "ac_adapter.present",
-					     &err);
+	if ((udis = libhal_find_device_by_capability(halctx, "ac_adapter", &num_devices, &err)) != NULL) {
+		for (i = 0; udis[i] != NULL; i++) {
+			LIBHAL_FREE_DBUS_ERROR (&err);
+
+			if (libhal_device_property_exists (halctx, udis[i], "ac_adapter.present", &err)) {
+				AC = libhal_device_get_property_bool (halctx, udis[i], "ac_adapter.present", &err);
+				break; /* we found one AC device, leave the for-loop */
+			}
+		}
+		libhal_free_string_array(udis);
+	}
 
 	if (dbus_message_is_method_call (message, 
 					 "org.freedesktop.Hal.Device.LaptopPanel", 
@@ -196,6 +233,8 @@ filter_function (DBusConnection *connection, DBusMessage *message, void *userdat
 		
 		HAL_DEBUG (("Received SetBrightness DBus call"));
 
+		LIBHAL_FREE_DBUS_ERROR (&err);
+
 		if (dbus_message_get_args (message, 
 					   &err,
 					   DBUS_TYPE_INT32, &brightness,
@@ -250,6 +289,8 @@ filter_function (DBusConnection *connection, DBusMessage *message, void *userdat
 		}
 	}	
 error:
+	LIBHAL_FREE_DBUS_ERROR (&err);
+
 	if (reply != NULL)
 		dbus_message_unref (reply);
 	
@@ -260,6 +301,7 @@ int
 main (int argc, char *argv[])
 {
  	DBusError err;
+	int retval = 0;
 
 	setup_logger ();
 
@@ -268,18 +310,20 @@ main (int argc, char *argv[])
 	HAL_DEBUG (("udi=%s", udi));
 	if (udi == NULL) {
 		HAL_ERROR (("No device specified"));
-		return -2;
+		retval = -2;
+		goto out;
 	}
 
 	dbus_error_init (&err);
 	if ((halctx = libhal_ctx_init_direct (&err)) == NULL) {
 		HAL_ERROR (("Cannot connect to hald"));
-		return -3;
+		retval = -3;
+		goto out;
 	}
 
-	dbus_error_init (&err);
 	if (!libhal_device_addon_is_ready(halctx, udi, &err)) {
-		return -4;
+		retval = -4;
+		goto out;
 	}
 
 	conn = libhal_ctx_get_dbus_connection (halctx);
@@ -289,9 +333,14 @@ main (int argc, char *argv[])
 
 	read_backlight (TRUE); /* Fill min- & maxValue with the correct values */
 
+#ifdef HAVE_SMBIOS_2_0_3
+	check_bios_password(); /* Find out about our BIOS pass capabilities */
+#endif
+
 	if (maxValue == 0) {
 		HAL_ERROR (("This machine don't support set brightness."));
-		return -5;
+		retval = -5;
+		goto out;
 	}
 	
 	if (!libhal_device_set_property_int (halctx, 
@@ -300,7 +349,8 @@ main (int argc, char *argv[])
 					    maxValue + 1,
 					    &err)) {
 		HAL_ERROR (("Could not set 'laptop_panel.numlevels' to %d", maxValue));
-		return -4;
+		retval = -4;
+		goto out;
 	}
 	HAL_DEBUG (("laptop_panel.numlevels is set to %d", maxValue + 1));
 
@@ -317,12 +367,25 @@ main (int argc, char *argv[])
 					    "    </method>\n",
 					    &err)) {
 		HAL_ERROR (("Cannot claim interface 'org.freedesktop.Hal.Device.LaptopPanel'"));
-		return -4;
+		retval = -4;
+		goto out;
 	}
 	
 	main_loop = g_main_loop_new (NULL, FALSE);
 	g_main_loop_run (main_loop);
 	return 0;
+
+out: 
+
+	LIBHAL_FREE_DBUS_ERROR (&err);
+
+	if (halctx != NULL) {
+		libhal_ctx_shutdown (halctx, &err);
+		LIBHAL_FREE_DBUS_ERROR (&err);
+		libhal_ctx_free (halctx);
+	}
+
+	return retval;
 }
 
 
