@@ -50,23 +50,26 @@
 #include "hotplug.h"
 
 /** Queue of ordered hotplug events */
-GQueue *hotplug_event_queue;
+static GQueue *hotplug_event_queue = NULL;
+
+/* Flag indicating if the queue should be reprocessed from the start */
+static gboolean hotplug_event_queue_restart = FALSE;
 
 /** List of HotplugEvent objects we are currently processing */
-GSList *hotplug_events_in_progress = NULL;
+static GList *hotplug_events_in_progress = NULL;
 
 void
 hotplug_event_end (void *end_token)
 {
 	HotplugEvent *hotplug_event = (HotplugEvent *) end_token;
 
-	hotplug_events_in_progress = g_slist_remove (hotplug_events_in_progress, hotplug_event);
+	hotplug_events_in_progress = g_list_remove (hotplug_events_in_progress, hotplug_event);
 
-	if (hotplug_event->free_function != NULL) {
-		hotplug_event->free_function (hotplug_event);
-	} else {
-		g_free (hotplug_event);
-	}
+	g_slice_free (HotplugEvent, hotplug_event);
+
+	/* An event is removed. So we need to restart from the beginning of the queue
+	 * as some events are ready to run now */
+	hotplug_event_queue_restart = TRUE;
 }
 
 void 
@@ -75,7 +78,7 @@ hotplug_event_reposted (void *end_token)
 	HotplugEvent *hotplug_event = (HotplugEvent *) end_token;
 
 	hotplug_event->reposted = TRUE;
-	hotplug_events_in_progress = g_slist_remove (hotplug_events_in_progress, hotplug_event);
+	hotplug_events_in_progress = g_list_remove (hotplug_events_in_progress, hotplug_event);
 }
 
 static void
@@ -113,7 +116,7 @@ hotplug_event_begin_sysfs (HotplugEvent *hotplug_event)
 	if (hotplug_event->type == HOTPLUG_EVENT_SYSFS && d != NULL) {
 		HotplugEventType type;
 
-		type = hal_device_property_get_int (d, "linux.hotplug_type");
+		type = (HotplugEventType) hal_device_property_get_int (d, "linux.hotplug_type");
 		if (type == HOTPLUG_EVENT_SYSFS_DEVICE) {
 			HAL_INFO (("%s is a device (store)", hotplug_event->sysfs.sysfs_path));
 			hotplug_event->type = HOTPLUG_EVENT_SYSFS_DEVICE;
@@ -314,7 +317,7 @@ hotplug_event_begin (HotplugEvent *hotplug_event)
 void 
 hotplug_event_enqueue (HotplugEvent *hotplug_event)
 {
-	if (hotplug_event_queue == NULL)
+	if (G_UNLIKELY (hotplug_event_queue == NULL))
 		hotplug_event_queue = g_queue_new ();
 
 	g_queue_push_tail (hotplug_event_queue, hotplug_event);
@@ -323,40 +326,185 @@ hotplug_event_enqueue (HotplugEvent *hotplug_event)
 void 
 hotplug_event_enqueue_at_front (HotplugEvent *hotplug_event)
 {
-	if (hotplug_event_queue == NULL)
+	if (G_UNLIKELY (hotplug_event_queue == NULL))
 		hotplug_event_queue = g_queue_new ();
 
 	g_queue_push_head (hotplug_event_queue, hotplug_event);
+
+	/* New event added at the start, restart processing of the queue from the
+	 * start */
+	hotplug_event_queue_restart = TRUE;
 }
 
+static gboolean
+compare_sysfspath(const char *running, const char *waiting)
+{
+	int i;
+
+	for (i = 0; i < HAL_PATH_MAX; i++) {
+		/* identical device event found */
+		if (running[i] == '\0' && waiting[i] == '\0')
+			return TRUE;
+
+		/* parent device event found */
+		if (running[i] == '\0' && waiting[i] == '/')
+			return TRUE;
+
+		/* child device event found */
+		if (running[i] == '/' && waiting[i] == '\0')
+			return TRUE;
+
+		/* no matching event */
+		if (running[i] != waiting[i])
+			break;
+	}
+
+	return FALSE;
+}
+
+/*
+ * Returns TRUE if @eventl depends on @eventr
+ */
+static gboolean
+compare_event (HotplugEvent *eventl, HotplugEvent *eventr)
+{
+	if (*eventl->sysfs.sysfs_path_old != '\0')
+		if (strcmp (eventr->sysfs.sysfs_path_old, eventl->sysfs.sysfs_path_old) == 0)
+			return TRUE;
+	return compare_sysfspath (eventr->sysfs.sysfs_path, eventl->sysfs.sysfs_path);
+}
+
+/*
+ * Returns TRUE if @hotplug_event depends on any event in @events
+ */
+static gboolean
+compare_events (HotplugEvent *hotplug_event, GList *events)
+{
+	GList *lp;
+	HotplugEvent *loop_event;
+
+	switch (hotplug_event->type) {
+
+	/* explicit fallthrough */
+	case HOTPLUG_EVENT_SYSFS:
+	case HOTPLUG_EVENT_SYSFS_DEVICE:
+	case HOTPLUG_EVENT_SYSFS_BLOCK:
+
+		for (lp = events; lp; lp = g_list_next (lp)) {
+			loop_event = (HotplugEvent*) lp->data;
+			/* skip ourselves and all later events*/
+			if (loop_event->sysfs.seqnum >= hotplug_event->sysfs.seqnum) {
+				HAL_DEBUG (("event %s: skip ourselves and all later events", hotplug_event->sysfs.sysfs_path));
+				break;
+			}
+			if (compare_event (hotplug_event, loop_event)) {
+				HAL_DEBUG (("event %s dependant on %s", hotplug_event->sysfs.sysfs_path, loop_event->sysfs.sysfs_path));
+				return TRUE;
+			} else if (hotplug_event->sysfs.is_dm_device) {
+				if ((loop_event->type == HOTPLUG_EVENT_SYSFS_BLOCK) && !loop_event->sysfs.is_dm_device) {
+					HAL_DEBUG (("event %s is dm-device, have at least one (%s) non-dm block device in queue -> held event.",
+						    hotplug_event->sysfs.sysfs_path, loop_event->sysfs.sysfs_path ));
+					return TRUE;
+				}
+			}
+		}
+		return FALSE;
+
+	default:
+		return FALSE;
+	}
+}
+
+/*
+ * Returns TRUE if @hotplug_event depends on any running event in @events
+ * This function checks if there is a running remove/add event for the same 
+ * sysfs_path/device. (for more see fd.o#23060)
+ */
+static gboolean
+compare_events_running (HotplugEvent *hotplug_event, GList *events)
+{
+	GList *lp;
+	HotplugEvent *loop_event;
+
+	switch (hotplug_event->type) {
+
+	/* explicit fallthrough */
+	case HOTPLUG_EVENT_SYSFS:
+	case HOTPLUG_EVENT_SYSFS_DEVICE:
+	case HOTPLUG_EVENT_SYSFS_BLOCK:
+
+		for (lp = events; lp; lp = g_list_next (lp)) {
+			loop_event = (HotplugEvent*) lp->data;
+			/* skip ourselves and all later events*/
+			if (!strcmp (hotplug_event->sysfs.sysfs_path, loop_event->sysfs.sysfs_path)) {
+				if (loop_event->action != hotplug_event->action) {
+					HAL_DEBUG (("there is still a event running for this device, wait!"));
+					return TRUE;
+				}
+
+			}
+		}
+		return FALSE;
+
+	default:
+		return FALSE;
+	}
+}
+
+
+
 void 
 hotplug_event_process_queue (void)
 {
 	HotplugEvent *hotplug_event;
+	GList *lp, *lp2;
+	static gboolean processing = FALSE;
 
-        while (hotplug_events_in_progress != NULL ||
-		(hotplug_event_queue != NULL &&
-		 !g_queue_is_empty (hotplug_event_queue))) {
+	if (G_UNLIKELY (hotplug_event_queue == NULL))
+		return;
 
-		/* do not process events if some other event is in progress 
-		 *
-		 * TODO: optimize so we can do add events in parallel by inspecting the
-		 *       wait_for_sysfs_path parameter and hotplug_events_in_progress list
-		 */
-		if (hotplug_events_in_progress != NULL && g_slist_length (hotplug_events_in_progress) > 0)
-			goto out;
+	if (processing)
+		return;
+	
+	processing = TRUE;
 
-		hotplug_event = g_queue_pop_head (hotplug_event_queue);
-		if (hotplug_event == NULL)
-			goto out;
+	lp = hotplug_event_queue->head;
+	while (lp != NULL) {
+		hotplug_event = lp->data;
 
-		hotplug_events_in_progress = g_slist_append (hotplug_events_in_progress, hotplug_event);
-		hotplug_event_begin (hotplug_event);
+		if (hotplug_event->action == HOTPLUG_ACTION_ADD)
+			HAL_DEBUG (("checking ADD event %s", hotplug_event->sysfs.sysfs_path));
+		else if (hotplug_event->action == HOTPLUG_ACTION_REMOVE)
+			HAL_DEBUG (("checking REMOVE event %s", hotplug_event->sysfs.sysfs_path));
+		else 
+			HAL_DEBUG (("checking event %s, action: %d", hotplug_event->sysfs.sysfs_path, hotplug_event->action));
+
+		if (!compare_events (hotplug_event, hotplug_event_queue->head) && 
+		    !compare_events (hotplug_event, hotplug_events_in_progress) &&
+		    !compare_events_running (hotplug_event, hotplug_events_in_progress)) {
+			lp2 = lp->prev;
+			g_queue_unlink(hotplug_event_queue, lp);
+			hotplug_events_in_progress = g_list_concat (hotplug_events_in_progress, lp);
+			hotplug_event_begin (hotplug_event);
+			if (lp2 == NULL || hotplug_event_queue_restart) {
+				lp = hotplug_event_queue->head;
+				hotplug_event_queue_restart = FALSE;
+			} else {
+				lp = g_list_next(lp2);
+			}
+		} else {
+			HAL_DEBUG (("event held back: %s", hotplug_event->sysfs.sysfs_path));
+			lp = g_list_next (lp);
+		}
 	}
+	HAL_DEBUG (("events queued = %d, events in progress = %d", hotplug_event_queue->length, g_list_length (hotplug_events_in_progress)));
 
-	hotplug_queue_now_empty ();
-out:
-        ;
+	processing = FALSE;
+
+	if (hotplug_event_queue->length == 0 && g_list_length (hotplug_events_in_progress) == 0) {
+		HAL_DEBUG(("Hotplug-queue empty now ... no hotplug events in progress"));
+		hotplug_queue_now_empty ();
+	}
 }
 
 gboolean 
@@ -409,6 +557,7 @@ hotplug_reprobe_generate_remove_events (HalDevice *d)
 		child = HAL_DEVICE (i->data);
 		hotplug_reprobe_generate_remove_events (child);
 	}
+	g_slist_free (childs);
 
 	/* then remove self */
 	HAL_INFO (("Generate remove event for udi %s", hal_device_get_udi (d)));
@@ -492,6 +641,7 @@ hotplug_reprobe_generate_add_events (HalDevice *d)
 		child = HAL_DEVICE (i->data);
 		hotplug_reprobe_generate_add_events (child);
 	}
+	g_slist_free (childs);
 }
 
 gboolean
