@@ -205,6 +205,7 @@ void                 libhal_drive_free                        (LibHalDrive *driv
 dbus_bool_t          libhal_drive_is_hotpluggable          (LibHalDrive      *drive);
 dbus_bool_t          libhal_drive_uses_removable_media     (LibHalDrive      *drive);
 dbus_bool_t          libhal_drive_is_media_detected        (LibHalDrive      *drive);
+dbus_bool_t          libhal_drive_is_media_detection_automatic (LibHalDrive      *drive);
 dbus_uint64_t        libhal_drive_get_size                 (LibHalDrive      *drive);
 dbus_uint64_t        libhal_drive_get_media_size           (LibHalDrive      *drive);
 const char          *libhal_drive_get_partition_scheme     (LibHalDrive      *drive);
@@ -280,7 +281,8 @@ typedef enum {
 	LIBHAL_VOLUME_DISC_TYPE_HDDVDROM    = 0x0d,
 	LIBHAL_VOLUME_DISC_TYPE_HDDVDR      = 0x0e,
 	LIBHAL_VOLUME_DISC_TYPE_HDDVDRW     = 0x0f,
-	LIBHAL_VOLUME_DISC_TYPE_MO          = 0x10
+	LIBHAL_VOLUME_DISC_TYPE_MO          = 0x10,
+	LIBHAL_VOLUME_DISC_TYPE_DVDR_DL     = 0x11
 } LibHalVolumeDiscType;
 
 LibHalVolume     *libhal_volume_from_udi                      (LibHalContext *hal_ctx, 
