@@ -48,7 +48,7 @@ usage (void)
 }
 
 
-void static
+static void
 unknown_closetray_error (const char *detail)
 {
 	fprintf (stderr, "org.freedesktop.Hal.Device.Storage.UnknownFailure\n");
@@ -146,7 +146,8 @@ main (int argc, char *argv[])
 	}
 
 	/* use handle_eject() with the closetray option */
-	handle_eject (hal_ctx, 
+	handle_eject (system_bus,
+                      hal_ctx, 
 		      libhal_drive_get_udi (drive),
 		      drive,
 		      libhal_drive_get_device_file (drive),
